# flake8: noqa: E501


class PrescriptionTreatmentType(object):
    """
    Constants for prescription treatment type.
    """

    ACUTE_PRESCRIBING = "0001"  # "one-off" prescriptions
    REPEAT_PRESCRIBING = "0002"  # may be re-issued by prescribing site
    REPEAT_DISPENSING = "0003"  # may be automatically reissued by Spine

    prescriptionTreatmentTypes = {
        ACUTE_PRESCRIBING: "Acute Prescription",
        REPEAT_PRESCRIBING: "Repeat Prescribing",
        REPEAT_DISPENSING: "Repeat Dispensing",
    }


class PrescriptionTypes(object):
    """
    Constants for prescription type.
    """

    # Translate prescription type codes to their text value
    prescriptionTypeCodes = {
        "0001": "GENERAL PRACTITIONER PRESCRIBING",
        "0002": "INTENTIONALLY LEFT BLANK",
        "0003": "NURSE PRACTITIONER PRESCRIBING",
        "0004": "HOSPITAL PRESCRIBING",
        "0006": "DENTAL PRESCRIBING",
        "0007": "SUPPLEMENTARY PRESCRIBER PRESCRIBING",
        "0009": "GENERAL PRACTITIONER PRESCRIBING: PRIVATE",
        "0012": "EXTENDED FORUMULARY PRESCRIBER",
        "0101": "PRIMARY CARE PRESCRIBER - MEDICAL PRESCRIBER",
        "0102": "GENERAL PRACTITIONER PRESCRIBING - TRAINEE DOCTOR/GP REGISTRAR",
        "0103": "GENERAL PRACTITIONER PRESCRIBING - DEPUTISING SERVICES",
        "0104": "PRIMARY CARE PRESCRIBER - NURSE INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0105": "PRIMARY CARE PRESCRIBER - COMMUNITY PRACTITIONER NURSE PRESCRIBER",
        "0106": "GENERAL PRACTITIONER PRESCRIBING - PCT EMPLOYED NURSE INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0107": "GENERAL PRACTITIONER PRESCRIBING - PCT EMPLOYED COMMUNITY PRACTITIONER NURSE PRESCRIBER",
        "0108": "PRIMARY CARE PRESCRIBER - PHARMACIST INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0109": "GENERAL PRACTITIONER PRESCRIBING - PCT EMPLOYED PHARMACIST PRESCRIBER",
        "0113": "PRIMARY CARE PRESCRIBER - OPTOMETRIST INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0114": "PRIMARY CARE PRESCRIBER - PODIATRIST/CHIROPODIST INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0116": "PRIMARY CARE PRESCRIBER - RADIOGRAPHER INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0117": "PRIMARY CARE PRESCRIBER - PHYSIOTHERAPIST INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0119": "GENERAL PRACTITIONER PRESCRIBING - PCT EMPLOYED PODIATRIST/CHIROPODIST",
        "0120": "GENERAL PRACTITIONER PRESCRIBING - PCT EMPLOYED OPTOMETRIST",
        "0121": "GENERAL PRACTITIONER PRESCRIBING - PCT EMPLOYED RADIOGRAPHER",
        "0122": "GENERAL PRACTITIONER PRESCRIBING - PCT EMPLOYED PHYSIOTHERAPIST",
        "0123": "PRIMARY CARE PRESCRIBER - HOSPITAL PRESCRIBER",
        "0124": "PRIMARY CARE PRESCRIBER - DIETICIAN SUPPLEMENTARY PRESCRIBER",
        "0125": "PRIMARY CARE PRESCRIBER - PARAMEDIC INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0304": "NURSE PRACTITIONER - PRACTICE EMPLOYED NURSE INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0305": "NURSE PRACTITIONER - PRACTICE EMPLOYED COMMUNITY PRACTITIONER NURSE PRESCRIBER",
        "0306": "NURSE PRACTITIONER - PRACTICE EMPLOYED NURSE INDEPENDENT/SUPPLEMENTARY PRESCRIBER",
        "0307": "NURSE PRACTITIONER - PRACTICE EMPLOYED COMMUNITY PRACTITIONER NURSE PRESCRIBER",
        "0406": "HOSPITAL PRESCRIBING - HOSPITAL PRESCRIBER",
        "0607": "DENTAL PRESCRIBING - DENTIST",
        "0708": "SUPPLEMENTARY PRESCRIBING - PRACTICE EMPLOYED PHARMACIST",
        "0709": "SUPPLEMENTARY PRESCRIBING - PCT EMPLOYED PHARMACIST",
        "0713": "SUPPLEMENTARY PRESCRIBING - PRACTICE EMPLOYED OPTOMETRIST",
        "0714": "SUPPLEMENTARY PRESCRIBING - PRACTICE EMPLOYED PODIATRIST/CHIROPODIST",
        "0716": "SUPPLEMENTARY PRESCRIBING - PRACTICE EMPLOYED RADIOGRAPHER",
        "0717": "SUPPLEMENTARY PRESCRIBING - PRACTICE EMPLOYED PHYSIOTHERAPIST",
        "0718": "SUPPLEMENTARY PRESCRIBING - PCT EMPLOYED OPTOMETRIST",
        "0719": "SUPPLEMENTARY PRESCRIBING - PCT EMPLOYED PODIATRIST/CHIROPODIST",
        "0721": "SUPPLEMENTARY PRESCRIBING - PCT EMPLOYED RADIOGRAPHER",
        "0722": "SUPPLEMENTARY PRESCRIBING - PCT EMPLOYED PHYSIOTHERAPIST",
        "0901": "PRIVATE PRESCRIBING - GP",
        "0904": "PRIVATE PRESCRIBING - NURSE PRESCRIBING",
        "0908": "PRIVATE PRESCRIBING - PHARMACIST PRESCRIBING",
        "0913": "PRIVATE PRESCRIBING - OPTOMETRIST",
        "0914": "PRIVATE PRESCRIBING - PODIATRIST/CHIROPODIST",
        "0915": "PRIVATE PRESCRIBING - PHYSIOTHERAPIST",
        "0916": "PRIVATE PRESCRIBING - RADIOGRAPHER",
        "1004": "Outpatient Community Prescriber - Nurse Independent Supplementary prescriber - FP10SS (HP) Hospital outpatient prescriptions dispensed in a community pharmacy",
        "1005": "Outpatient Community Prescriber - Community Practitioner Nurse prescriber - FP10SS (HP) Hospital outpatient prescriptions dispensed in a community pharmacy",
        "1008": "Outpatient Community Prescriber - Pharmacist Independent Supplementary prescriber - FP10SS (HP) Hospital outpatient prescriptions dispensed in a community pharmacy",
        "1013": "Outpatient Community Prescriber - Optometrist Independent Supplementary prescriber - FP10SS (HP) Hospital outpatient prescriptions dispensed in a community pharmacy",
        "1014": "Outpatient Community Prescriber - Podiatrist Chiropodist Independent Supplementary prescriber - FP10SS (HP) Hospital outpatient prescriptions dispensed in a community pharmacy",
        "1016": "Outpatient Community Prescriber - Radiographer Independent Supplementary prescriber - FP10SS (HP) Hospital outpatient prescriptions dispensed in a community pharmacy",
        "1017": "Outpatient Community Prescriber - Physiotherapist Independent Supplementary prescriber - FP10SS (HP) Hospital outpatient prescriptions dispensed in a community pharmacy",
        "1024": "Outpatient Community Prescriber - Dietician Supplementary prescriber - FP10SS (HP) Hospital outpatient prescriptions dispensed in a community pharmacy",
        "1025": "Outpatient Community Prescriber - Paramedic Independent Supplementary prescriber - FP10SS (HP) Hospital outpatient prescriptions dispensed in a community pharmacy",
        "1104": "Outpatient Hospital Pharmacy Prescriber - Nurse Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed in their own hospital pharmacy",
        "1105": "Outpatient Hospital Pharmacy Prescriber - Community Practitioner Nurse prescriber - NON- FP10 Hospital outpatient prescriptions dispensed in their own hospital pharmacy",
        "1108": "Outpatient Hospital Pharmacy Prescriber - Pharmacist Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed in their own hospital pharmacy",
        "1113": "Outpatient Hospital Pharmacy Prescriber - Optometrist Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed in their own hospital pharmacy",
        "1114": "Outpatient Hospital Pharmacy Prescriber - Podiatrist Chiropodist Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed in their own hospital pharmacy",
        "1116": "Outpatient Hospital Pharmacy Prescriber - Radiographer Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed in their own hospital pharmacy",
        "1117": "Outpatient Hospital Pharmacy Prescriber - Physiotherapist Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed in their own hospital pharmacy",
        "1124": "Outpatient Hospital Pharmacy Prescriber - Dietician Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed in their own hospital pharmacy",
        "1125": "Outpatient Hospital Pharmacy Prescriber - Paramedic Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed in their own hospital pharmacy",
        "1204": "Outpatient Homecare Prescriber - Nurse Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed by Homecare",
        "1205": "Outpatient Homecare Prescriber - Community Practitioner Nurse prescriber - NON- FP10 Hospital outpatient prescriptions dispensed by Homecare",
        "1208": "Outpatient Homecare Prescriber - Pharmacist Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed by Homecare",
        "1213": "Outpatient Homecare Prescriber - Optometrist Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed by Homecare",
        "1214": "Outpatient Homecare Prescriber - Podiatrist Chiropodist Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed by Homecare",
        "1216": "Outpatient Homecare Prescriber - Radiographer Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed by Homecare",
        "1217": "Outpatient Homecare Prescriber - Physiotherapist Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed by Homecare",
        "1224": "Outpatient Homecare Prescriber - Dietician Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed by Homecare",
        "1225": "Outpatient Homecare Prescriber - Paramedic Independent Supplementary prescriber - NON- FP10 Hospital outpatient prescriptions dispensed by Homecare",
        "1001": "Outpatient Community Prescriber - Medical Prescriber",
        "1101": "Outpatient Pharmacy Prescriber - Medical Prescriber",
        "1201": "Outpatient Homecare Prescriber - Medical Prescriber",
        # WELSH CODES
        "0201": "Primary Care Prescriber - Medical Prescriber (Wales)",
        "0204": "Primary Care Prescriber - Nurse Independent/Supplementary prescriber (Wales)",
        "0205": "Primary Care Prescriber - Community Practitioner Nurse prescriber (Wales)",
        "0208": "Primary Care Prescriber - Pharmacist Independent/Supplementary prescriber (Wales)",
        "0213": "Primary Care Prescriber - Optometrist Independent/Supplementary prescriber (Wales)",
        "0214": "Primary Care Prescriber - Podiatrist/Chiropodist Independent/Supplementary prescriber (Wales)",
        "0216": "Primary Care Prescriber - Radiographer Independent/Supplementary prescriber (Wales)",
        "0217": "Primary Care Prescriber - Physiotherapist Independent/Supplementary prescriber (Wales)",
        "0224": "Primary Care Prescriber - Dietician Supplementary prescriber (Wales)",
        "0225": "Primary Care Prescriber - Paramedic Independent/Supplementary prescriber (Wales)",
        "2001": "Outpatient Community Prescriber - Medical Prescriber (Wales)",
        "2004": "Outpatient Community Prescriber - Nurse Independent/Supplementary prescriber (Wales)",
        "2005": "Outpatient Community Prescriber - Community Practitioner Nurse prescriber (Wales)",
        "2008": "Outpatient Community Prescriber - Pharmacist Independent/Supplementary prescriber (Wales)",
        "2013": "Outpatient Community Prescriber - Optometrist Independent/Supplementary prescriber (Wales)",
        "2014": "Outpatient Community Prescriber - Podiatrist/Chiropodist Independent/Supplementary (Wales)",
        "2016": "Outpatient Community Prescriber - Radiographer Independent/Supplementary prescriber (Wales)",
        "2017": "Outpatient Community Prescriber - Physiotherapist Independent/Supplementary prescriber (Wales)",
        "2024": "Outpatient Community Prescriber - Dietician Supplementary prescriber (Wales)",
        "2025": "Outpatient Community Prescriber - Paramedic Independent/Supplementary prescriber (Wales)",
        "0707": "Dental Prescribing - Dentist (Wales)",
        # ISLE OF MANN CODES
        "0501": "Primary Care Prescriber - Medical Prescriber (IOM)",
        "0504": "Primary Care Prescriber - Nurse Independent/Supplementary prescriber (IOM)",
        "0505": "Primary Care Prescriber - Community Practitioner Nurse prescriber (IOM)",
        "0508": "Primary Care Prescriber - Pharmacist Independent/Supplementary prescriber (IOM)",
        "0513": "Primary Care Prescriber - Optometrist Independent/Supplementary prescriber (IOM)",
        "0514": "Primary Care Prescriber - Podiatrist/Chiropodist Independent/Supplementary prescriber (IOM)",
        "0516": "Primary Care Prescriber - Radiographer Independent/Supplementary prescriber (IOM)",
        "0517": "Primary Care Prescriber - Physiotherapist Independent/Supplementary prescriber (IOM)",
        "0524": "Primary Care Prescriber - Dietician Supplementary prescriber (IOM)",
        "0525": "Primary Care Prescriber - Paramedic Independent/Supplementary prescriber (IOM)",
        "5001": "Outpatient Community Prescriber - Medical Prescriber (IOM)",
        "5004": "Outpatient Community Prescriber - Nurse Independent/Supplementary prescriber (IOM)",
        "5005": "Outpatient Community Prescriber - Community Practitioner Nurse prescriber (IOM)",
        "5008": "Outpatient Community Prescriber - Pharmacist Independent/Supplementary prescriber (IOM)",
        "5013": "Outpatient Community Prescriber - Optometrist Independent/Supplementary prescriber (IOM)",
        "5014": "Outpatient Community Prescriber - Podiatrist/Chiropodist Independent/Supplementary (IOM)",
        "5016": "Outpatient Community Prescriber - Radiographer Independent/Supplementary prescriber (IOM)",
        "5017": "Outpatient Community Prescriber - Physiotherapist Independent/Supplementary prescriber (IOM)",
        "5024": "Outpatient Community Prescriber - Dietician Supplementary prescriber (IOM)",
        "5025": "Outpatient Community Prescriber - Paramedic Independent/Supplementary prescriber (IOM)",
    }



import datetime

from eps_spine_shared.common.prescription import fields
from eps_spine_shared.common.prescription.claim import PrescriptionClaim
from eps_spine_shared.common.prescription.line_item import PrescriptionLineItem
from eps_spine_shared.common.prescription.statuses import PrescriptionStatus
from eps_spine_shared.nhsfundamentals.timeutilities import TimeFormats


class PrescriptionIssue(object):
    """
    Wrapper class to simplify interacting with an issue (instance) portion of a prescription record.

    Note: the correct domain terminology is "issue", however there are legacy references
    to "instance" in the code and database records.
    """

    def __init__(self, issue_dict):
        """
        Constructor.

        :type issue_dict: dict
        """
        self._issue_dict = issue_dict

    @property
    def number(self):
        """
        The number of this issue.

        :rtype: int
        """
        # Note: the number is stored as a string, so we need to convert
        number = int(self._issue_dict[fields.FIELD_INSTANCE_NUMBER])
        return number

    @property
    def status(self):
        """
        The status code of the issue

        :rtype: str
        """
        return self._issue_dict[fields.FIELD_PRESCRIPTION_STATUS]

    @status.setter
    def status(self, new_status):
        """
        The status code of the issue

        NOTE: this does not update the previous status - use update_status() to do that
        PAB - should we be using update_status() in places we are using this?
        :type new_status: str
        """
        self._issue_dict[fields.FIELD_PRESCRIPTION_STATUS] = new_status

    @property
    def completion_date_str(self):
        """
        The issue completion date as a YYYYMMDD string, if available.

        :rtype: str or None
        """
        completion_date_str = self._issue_dict[fields.FIELD_COMPLETION_DATE]
        if not completion_date_str:
            return None
        return completion_date_str

    def expire(self, expired_at_time, parent_prescription):
        """
        Update the issue and all its line items to be expired.

        :type expired_at_time: datetime.datetime
        :type parent_prescription: PrescriptionRecord
        """

        currentStatus = self.status

        # update the issue status, if appropriate
        if currentStatus not in PrescriptionStatus.EXPIRY_IMMUTABLE_STATES:
            newStatus = PrescriptionStatus.EXPIRY_LOOKUP[currentStatus]
            self.update_status(newStatus, parent_prescription)

            if currentStatus in PrescriptionStatus.UNACTIONED_STATES:
                parent_prescription.log_object.write_log(
                    "EPS0616",
                    None,
                    {
                        "internalID": parent_prescription.internal_id,
                        "previousStatus": currentStatus,
                        "releaseVersion": parent_prescription.get_release_version(),
                        "prescriptionID": str(parent_prescription.return_prescription_id()),
                    },
                )

        # make sure all the line items are expired as well
        for lineItem in self.line_items:
            lineItem.expire(parent_prescription)

        parent_prescription.log_object.write_log(
            "EPS0403",
            None,
            {
                "internalID": parent_prescription.internal_id,
            },
        )

        # PAB: this will update the completion time of issues that are
        # already in EXPIRY_IMMUTABLE_STATES (ie. already completed) - is
        # this correct, or should this be guarded in the above if statement?
        self.mark_completed(expired_at_time, parent_prescription)

    def mark_completed(self, completion_datetime, parent_prescription):
        """
        Update the completion date of this issue.

        :type completion_datetime: datetime.datetime
        :type parent_prescription: PrescriptionRecord
        """
        current_completion_date_str = self.completion_date_str

        new_completion_date_str = completion_datetime.strftime(TimeFormats.STANDARD_DATE_FORMAT)
        self._issue_dict[fields.FIELD_COMPLETION_DATE] = new_completion_date_str

        parent_prescription.log_attribute_change(
            fields.FIELD_COMPLETION_DATE,
            (current_completion_date_str or ""),
            new_completion_date_str,
            None,
        )

    @property
    def expiry_date_str(self):
        """
        The issue expiry date as a YYYYMMDD string.

        :rtype: str
        """
        return self._issue_dict[fields.FIELD_EXPIRY_DATE]

    @property
    def line_items(self):
        """
        The line items for this issue.

        :rtype: list(PrescriptionLineItem)
        """
        line_item_dicts = self._issue_dict[fields.FIELD_LINE_ITEMS]
        # wrap the dicts to add convenience methods
        line_items = [PrescriptionLineItem(d) for d in line_item_dicts]
        return line_items

    @property
    def claim(self):
        """
        The claim information for this issue.

        :rtype: PrescriptionClaim
        """
        claim_dict = self._issue_dict[fields.FIELD_CLAIM]
        return PrescriptionClaim(claim_dict)

    def update_status(self, new_status, parent_prescription):
        """
        Update the issue status, and record the previous status.

        :type new_status: str
        """
        currentStatus = self.status
        self._issue_dict[fields.FIELD_PREVIOUS_STATUS] = currentStatus
        self._issue_dict[fields.FIELD_PRESCRIPTION_STATUS] = new_status
        parent_prescription.log_attribute_change(
            fields.FIELD_PRESCRIPTION_STATUS, currentStatus, new_status, None
        )

    @property
    def dispensing_organization(self):
        """
        Dispensing organization for this issue.

        :rtype: str
        """
        dispense_dict = self._issue_dict[fields.FIELD_DISPENSE]
        return dispense_dict[fields.FIELD_DISPENSING_ORGANIZATION]

    @property
    def last_dispense_date(self):
        """
        Dispensing date for this issue.

        :rtype: str
        """
        dispense_dict = self._issue_dict[fields.FIELD_DISPENSE]
        return dispense_dict[fields.FIELD_LAST_DISPENSE_DATE]

    @property
    def last_dispense_notification_msg_ref(self):
        """
        Last Dispense Notification MsgRef for this issue.

        :rtype: str
        """
        dispense_dict = self._issue_dict[fields.FIELD_DISPENSE]
        return dispense_dict[fields.FIELD_LAST_DISPENSE_NOTIFICATION_MSG_REF]

    def clear_dispensing_organisation(self):
        """
        Clear the dispensing organisation from this instance.
        """
        dispense_dict = self._issue_dict[fields.FIELD_DISPENSE]
        dispense_dict[fields.FIELD_DISPENSING_ORGANIZATION] = None

    @property
    def dispense_window_low_date(self):
        """
        Dispense window low date

        :rtype: datetime or None
        """
        low_date_str = self._issue_dict.get(fields.FIELD_DISPENSE_WINDOW_LOW_DATE)
        if not low_date_str:
            return None
        return datetime.datetime.strptime(low_date_str, TimeFormats.STANDARD_DATE_FORMAT)

    def has_active_line_item(self):
        """
        See if this instance has any active line items.

        :rtype: bool
        """
        return any(lineItem.is_active() for lineItem in self.line_items)

    def get_line_item_by_id(self, line_item_id):
        """
        Get a particular line item by its ID.

        Raises a KeyError if no item can be found.

        :type line_item_id: str
        :rtype: PrescriptionLineItem
        """
        for lineItem in self.line_items:
            if lineItem.id == line_item_id:
                return lineItem

        raise KeyError("Could not find line item '%s'" % line_item_id)

    @property
    def release_date(self):
        """
        The releaseDate for this issue, if one is specified

        :rtype: str
        """
        release_date = self._issue_dict.get(fields.FIELD_RELEASE_DATE)
        return str(release_date)

    @property
    def next_activity(self):
        """
        The next activity for this issue, if one is specified.

        Note: some migrated prescriptions may not have a next activity specified,
        although this should hopefully be rectified. If so, we may be able to tighten
        up the return type.

        :rtype: str or None
        """
        next_activity_dict = self._issue_dict[fields.FIELD_NEXT_ACTIVITY]
        return next_activity_dict.get(fields.FIELD_ACTIVITY, None)

    @property
    def next_activity_date_str(self):
        """
        The next activity date for this issue, if one is specified.

        :rtype: str or None
        """
        next_activity_dict = self._issue_dict[fields.FIELD_NEXT_ACTIVITY]
        return next_activity_dict.get(fields.FIELD_DATE, None)

    @property
    def cancellations(self):
        """
        The cancellations for this issue.

        :rtype: list()
        """
        return self._issue_dict[fields.FIELD_CANCELLATIONS]

    def get_line_item_cancellations(self, line_item_id):
        """
        Get the cancellations for a particular line item.

        :type line_item_id: str
        :rtype: list()
        """
        return [
            c for c in self.cancellations if c[fields.FIELD_CANCEL_LINE_ITEM_REF] == line_item_id
        ]

    def get_line_item_first_cancellation_time(self, line_item_id):
        """
        Get the time of the first cancellation targetting a particular line item.

        :type line_item_id: str
        :rtype: str or None
        """
        cancellations = self.get_line_item_cancellations(line_item_id)
        cancellation_times = [c[fields.FIELD_CANCELLATION_TIME] for c in cancellations]

        if cancellations:
            return min(cancellation_times, key=lambda x: int(x))
        return None

    @property
    def release_request_msg_ref(self):
        """
        The release request message reference for this issue.

        :rtype: str
        """
        return self._issue_dict[fields.FIELD_RELEASE_REQUEST_MGS_REF]



from eps_spine_shared.common.prescription import fields


class PrescriptionClaim(object):
    """
    Wrapper class to simplify interacting with an issue claim portion of a prescription record.
    """

    def __init__(self, claim_dict):
        """
        Constructor.

        :type claim_dict: dict
        """
        self._claim_dict = claim_dict

    @property
    def received_date_str(self):
        """
        The date the claim was received.

        :rtype: str
        """
        return self._claim_dict[fields.FIELD_CLAIM_RECEIVED_DATE]

    @received_date_str.setter
    def received_date_str(self, value):
        """
        The date the claim was received.

        :type value: str
        """
        self._claim_dict[fields.FIELD_CLAIM_RECEIVED_DATE] = value

    def get_dict(self):
        """
        returns claim_dict
        """
        return self._claim_dict



class PrescriptionStatus(object):
    """
    Prescription states and related information
    """

    AWAITING_RELEASE_READY = "0000"
    TO_BE_DISPENSED = "0001"
    WITH_DISPENSER = "0002"
    WITH_DISPENSER_ACTIVE = "0003"
    EXPIRED = "0004"
    CANCELLED = "0005"
    DISPENSED = "0006"
    NOT_DISPENSED = "0007"
    CLAIMED = "0008"
    NO_CLAIMED = "0009"
    REPEAT_DISPENSE_FUTURE_INSTANCE = "9000"
    FUTURE_DATED_PRESCRIPTION = "9001"
    PENDING_CANCELLATION = "9005"

    PRESCRIPTION_DISPLAY_LOOKUP = {}
    PRESCRIPTION_DISPLAY_LOOKUP[AWAITING_RELEASE_READY] = "Awaiting Release Ready"
    PRESCRIPTION_DISPLAY_LOOKUP[TO_BE_DISPENSED] = "To Be Dispensed"
    PRESCRIPTION_DISPLAY_LOOKUP[WITH_DISPENSER] = "With Dispenser"
    PRESCRIPTION_DISPLAY_LOOKUP[WITH_DISPENSER_ACTIVE] = "With Dispenser - Active"
    PRESCRIPTION_DISPLAY_LOOKUP[EXPIRED] = "Expired"
    PRESCRIPTION_DISPLAY_LOOKUP[CANCELLED] = "Cancelled"
    PRESCRIPTION_DISPLAY_LOOKUP[DISPENSED] = "Dispensed"
    PRESCRIPTION_DISPLAY_LOOKUP[NOT_DISPENSED] = "Not Dispensed"
    PRESCRIPTION_DISPLAY_LOOKUP[CLAIMED] = "Claimed"
    PRESCRIPTION_DISPLAY_LOOKUP[NO_CLAIMED] = "No-Claimed"
    PRESCRIPTION_DISPLAY_LOOKUP[REPEAT_DISPENSE_FUTURE_INSTANCE] = "Repeat Dispense future instance"
    PRESCRIPTION_DISPLAY_LOOKUP[FUTURE_DATED_PRESCRIPTION] = "Prescription future instance"
    PRESCRIPTION_DISPLAY_LOOKUP[PENDING_CANCELLATION] = "Cancelled future instance"

    CANCELLABLE_STATES = [
        AWAITING_RELEASE_READY,
        TO_BE_DISPENSED,
        REPEAT_DISPENSE_FUTURE_INSTANCE,
        FUTURE_DATED_PRESCRIPTION,
    ]

    WITH_DISPENSER_STATES = [WITH_DISPENSER, WITH_DISPENSER_ACTIVE]

    ACTIVE_STATES = [AWAITING_RELEASE_READY, TO_BE_DISPENSED, WITH_DISPENSER, WITH_DISPENSER_ACTIVE]

    FUTURE_STATES = [FUTURE_DATED_PRESCRIPTION, REPEAT_DISPENSE_FUTURE_INSTANCE]

    COMPLETED_STATES = [EXPIRED, CANCELLED, DISPENSED, NOT_DISPENSED, CLAIMED, NO_CLAIMED]

    NOT_COMPLETED_STATES = [
        AWAITING_RELEASE_READY,
        TO_BE_DISPENSED,
        WITH_DISPENSER,
        WITH_DISPENSER_ACTIVE,
        FUTURE_DATED_PRESCRIPTION,
        REPEAT_DISPENSE_FUTURE_INSTANCE,
    ]

    INCLUDE_PERFORMER_STATES = [
        WITH_DISPENSER,
        WITH_DISPENSER_ACTIVE,
        DISPENSED,
        NOT_DISPENSED,
        CLAIMED,
        NO_CLAIMED,
    ]

    EXPIRY_IMMUTABLE_STATES = [EXPIRED, CANCELLED, DISPENSED, NOT_DISPENSED, CLAIMED, NO_CLAIMED]

    UNACTIONED_STATES = [
        AWAITING_RELEASE_READY,
        TO_BE_DISPENSED,
        WITH_DISPENSER,
        REPEAT_DISPENSE_FUTURE_INSTANCE,
        PENDING_CANCELLATION,
    ]

    ALL_VALID_STATES = [
        AWAITING_RELEASE_READY,
        TO_BE_DISPENSED,
        WITH_DISPENSER,
        WITH_DISPENSER_ACTIVE,
        EXPIRED,
        CANCELLED,
        DISPENSED,
        NOT_DISPENSED,
        CLAIMED,
        NO_CLAIMED,
        REPEAT_DISPENSE_FUTURE_INSTANCE,
        FUTURE_DATED_PRESCRIPTION,
        PENDING_CANCELLATION,
    ]

    EXPIRY_LOOKUP = {}
    EXPIRY_LOOKUP[AWAITING_RELEASE_READY] = EXPIRED
    EXPIRY_LOOKUP[TO_BE_DISPENSED] = EXPIRED
    EXPIRY_LOOKUP[WITH_DISPENSER] = EXPIRED
    EXPIRY_LOOKUP[WITH_DISPENSER_ACTIVE] = DISPENSED
    EXPIRY_LOOKUP[REPEAT_DISPENSE_FUTURE_INSTANCE] = EXPIRED
    EXPIRY_LOOKUP[FUTURE_DATED_PRESCRIPTION] = EXPIRED
    EXPIRY_LOOKUP[PENDING_CANCELLATION] = EXPIRED


class LineItemStatus(object):
    """
    Prescription line item states and related information
    """

    FULLY_DISPENSED = "0001"
    NOT_DISPENSED = "0002"
    PARTIAL_DISPENSED = "0003"
    NOT_DISPENSED_OWING = "0004"
    CANCELLED = "0005"
    EXPIRED = "0006"
    TO_BE_DISPENSED = "0007"
    WITH_DISPENSER = "0008"

    ITEM_CANCELLABLE_STATES = [TO_BE_DISPENSED]
    ITEM_WITH_DISPENSER_STATES = [WITH_DISPENSER, PARTIAL_DISPENSED]

    ACTIVE_STATES = [TO_BE_DISPENSED, WITH_DISPENSER, PARTIAL_DISPENSED, NOT_DISPENSED_OWING]

    INCLUDE_PERFORMER_STATES = [
        WITH_DISPENSER,
        PARTIAL_DISPENSED,
        FULLY_DISPENSED,
        NOT_DISPENSED,
        NOT_DISPENSED_OWING,
    ]

    ITEM_DISPLAY_LOOKUP = {}
    ITEM_DISPLAY_LOOKUP[FULLY_DISPENSED] = "Item fully dispensed"
    ITEM_DISPLAY_LOOKUP[NOT_DISPENSED] = "Item not dispensed"
    ITEM_DISPLAY_LOOKUP[PARTIAL_DISPENSED] = "Item dispensed - partial"
    ITEM_DISPLAY_LOOKUP[NOT_DISPENSED_OWING] = "Item not dispensed owing"
    ITEM_DISPLAY_LOOKUP[EXPIRED] = "Expired"
    ITEM_DISPLAY_LOOKUP[CANCELLED] = "Item Cancelled"
    ITEM_DISPLAY_LOOKUP[TO_BE_DISPENSED] = "To Be Dispensed"
    ITEM_DISPLAY_LOOKUP[WITH_DISPENSER] = "Item with dispenser"

    VALID_STATES = {}
    VALID_STATES[PrescriptionStatus.AWAITING_RELEASE_READY] = [CANCELLED, EXPIRED, TO_BE_DISPENSED]
    VALID_STATES[PrescriptionStatus.TO_BE_DISPENSED] = [CANCELLED, EXPIRED, TO_BE_DISPENSED]
    VALID_STATES[PrescriptionStatus.WITH_DISPENSER] = [CANCELLED, EXPIRED, WITH_DISPENSER]
    VALID_STATES[PrescriptionStatus.WITH_DISPENSER_ACTIVE] = [
        FULLY_DISPENSED,
        NOT_DISPENSED,
        PARTIAL_DISPENSED,
        NOT_DISPENSED_OWING,
        CANCELLED,
        EXPIRED,
        WITH_DISPENSER,
    ]
    VALID_STATES[PrescriptionStatus.EXPIRED] = [CANCELLED, EXPIRED]
    VALID_STATES[PrescriptionStatus.CANCELLED] = [CANCELLED, EXPIRED]
    VALID_STATES[PrescriptionStatus.DISPENSED] = [
        FULLY_DISPENSED,
        NOT_DISPENSED,
        CANCELLED,
        EXPIRED,
    ]
    VALID_STATES[PrescriptionStatus.NOT_DISPENSED] = [NOT_DISPENSED, CANCELLED, EXPIRED]
    VALID_STATES[PrescriptionStatus.CLAIMED] = [FULLY_DISPENSED, NOT_DISPENSED, CANCELLED, EXPIRED]
    VALID_STATES[PrescriptionStatus.NO_CLAIMED] = [
        FULLY_DISPENSED,
        NOT_DISPENSED,
        CANCELLED,
        EXPIRED,
    ]
    VALID_STATES[PrescriptionStatus.REPEAT_DISPENSE_FUTURE_INSTANCE] = [
        CANCELLED,
        EXPIRED,
        TO_BE_DISPENSED,
    ]
    VALID_STATES[PrescriptionStatus.FUTURE_DATED_PRESCRIPTION] = [
        CANCELLED,
        EXPIRED,
        TO_BE_DISPENSED,
    ]

    EXPIRY_IMMUTABLE_STATES = [FULLY_DISPENSED, NOT_DISPENSED, EXPIRED, CANCELLED]

    EXPIRY_LOOKUP = {}
    EXPIRY_LOOKUP[TO_BE_DISPENSED] = "0006"
    EXPIRY_LOOKUP[PARTIAL_DISPENSED] = "0001"
    EXPIRY_LOOKUP[NOT_DISPENSED_OWING] = "0002"
    EXPIRY_LOOKUP[WITH_DISPENSER] = "0006"



from typing import Dict


class MockLogObject(object):
    """
    Mock log object
    """

    def __init__(self, severity_threshold="INFO"):
        self.__expectations = set([])
        self._called_references = []
        self._log_records = []
        self._severity_threshold = severity_threshold
        self.logged_messages = []

    def write_log(
        self,
        log_reference="UTI9999",
        error_list=None,
        log_row_dict=None,
        severity_threshold_override=None,
        process_name=None,
    ):
        """
        Dummy write log just keeps a list of the logReferences
        """
        log_row_dict = dict(log_row_dict) if log_row_dict else {}
        self.logged_messages.append((log_reference, log_row_dict))
        log_record = {
            "logReference": log_reference,
            "errorList": error_list,
            "logRowDict": log_row_dict,
            "severityThresholdOverride": severity_threshold_override,
            "processName": process_name,
        }
        self._log_records.append(log_record)

        self._called_references.append(log_reference)

        if log_reference in self.__expectations:
            self.__expectations.remove(log_reference)

    # Alias for Spine compatibility
    writeLog = write_log

    def was_logged(self, log_reference):
        """
        Was a particular log reference logged?
        """
        return log_reference in self._called_references

    def was_value_logged(self, log_reference, key, expected_value):
        """
        Was a particular log key supplied as expected
        """
        for log_record in self.log_records:
            if log_record["logReference"] == log_reference:
                # Deliberately done like this so that if there are multiple logReferences that are the same with
                # different value.
                match = expected_value == log_record["logRowDict"][key]
                if match:
                    return True
        return False

    def was_value_not_logged(self, log_reference, key, expected_value):
        """
        Was a particular log key not supplied as expected
        """
        for log_record in self.log_records:
            if log_record["logReference"] == log_reference:
                # Deliberately done like this so that if there are multiple logReferences that are the same with
                # different value.
                match = expected_value == log_record["logRowDict"][key]
                if match:
                    return False
        return True

    def logged_value_occurrences(self, log_reference, key, expected_value):
        """
        Return the number of occurrences of a particular key and value
        """
        occurrences = 0
        for log_record in self.log_records:
            if log_record["logReference"] == log_reference:
                # Deliberately done like this so that if there are multiple logReferences that are the same with
                # different value.
                match = expected_value == log_record["logRowDict"][key]
                if match:
                    occurrences += 1
        return occurrences

    def was_multiple_value_logged(self, log_reference: str, key_values: Dict):
        """
        Was a particular log key supplied as expected with the expected values
        """
        for log_record in self.log_records:
            found_count = 0
            if log_record["logReference"] == log_reference:
                for key in key_values:
                    if log_record["logRowDict"][key] == key_values[key]:
                        found_count += 1
                if found_count == len(key_values):
                    return True
        return False

    def get_logged_value(self, log_reference, key):
        """
        Get logged value for given reference and key
        """
        for log_record in self.log_records:
            if log_record["logReference"] == log_reference:
                return log_record["logRowDict"][key]

    def get_log_occurrences(self, log_reference):
        """
        Gets a list of the args that were passed each time a specified message was logged
        """
        return [args for reference, args in self.logged_messages if reference == log_reference]

    def log_occurrence_count(self, log_reference):
        """
        Returns the number of times a logReference was logged.
        """
        return len(self.get_log_occurrences(log_reference))

    def add_expected_reference(self, expected_reference):
        """
        set the expected reference
        """
        self.__expectations.add(expected_reference)

    def expectations_satisfied(self):
        """
        has the expected log line been written
        """
        return len(self.__expectations) == 0

    def clear_expectations(self):
        """
        clear expectations
        """
        self.__expectations = set([])

    def clear(self):
        """
        Clear everything
        """
        self.__expectations = set([])
        self._called_references = []
        self._log_records = []
        self.logged_messages = []

    @property
    def called_references(self):
        """
        The called references
        """
        return self._called_references

    @called_references.setter
    def called_references(self, new_called_references):
        """
        Setter
        """
        self._called_references = new_called_references

    @property
    def log_records(self):
        """
        The logRecords
        """
        return self._log_records

    @property
    def severity_threshold(self):
        """
        The severity threshold (ref logging.py _SEVERITY_INPUT_MAP)
        """
        return self._severity_threshold



from eps_spine_shared.common.prescription.record import PrescriptionRecord


class RepeatPrescribeRecord(PrescriptionRecord):
    """
    Class defined to handle repeat prescribe prescriptions
    """

    def __init__(self, log_object, internal_id):
        """
        Allow the record_type attribute to be set
        """
        super(RepeatPrescribeRecord, self).__init__(log_object, internal_id)
        self.record_type = "RepeatPrescribe"
